@@ -13,6 +13,7 @@
  */
 #include <config.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <gdk/gdkx.h>
 #include <gtk/gtksignal.h>
@@ -251,7 +252,6 @@ create_plug (BonoboControl *control)
 	GtkWidget *plug;
 	
 	plug = bonobo_plug_new (0);
-	gtk_widget_show (plug);
 
 	g_object_ref (G_OBJECT (plug));
 	gtk_object_sink (GTK_OBJECT (plug));
@@ -265,6 +265,61 @@ create_plug (BonoboControl *control)
 	g_object_unref (G_OBJECT (plug));
 }
 
+#ifdef HAVE_GTK_MULTIHEAD
+static int
+parse_cookie (const CORBA_char *cookie)
+{
+	GString    *ident = NULL;
+	GString    *value = NULL;
+	const char *p;
+	char       *screen = NULL;
+	int         retval = -1;
+
+	for (p = cookie; *p && !screen; p++) {
+		switch (*p) {
+		case ',':
+			if (!ident || !value)
+				goto parse_failed;
+
+			if (strcmp (ident->str, "screen")) {
+				g_string_free (ident, TRUE); ident = NULL;
+				g_string_free (value, TRUE); value = NULL;
+				break;
+			}
+
+			screen = value->str;
+			break;
+		case '=':
+			if (!ident || value)
+				goto parse_failed;
+			value = g_string_new ("");
+			break;
+		default:
+			if (!ident)
+				ident = g_string_new ("");
+			
+			if (value)
+				g_string_append_c (value, *p);
+			else
+				g_string_append_c (ident, *p);
+			break;
+		}
+	}
+
+	if (ident && value && !strcmp (ident->str, "screen"))
+		screen = value->str;
+
+	if (screen)
+		retval = atoi (screen);
+
+parse_failed:
+	if (ident) g_string_free (ident, TRUE);
+	if (value) g_string_free (value, TRUE);
+
+	return retval;
+}
+#endif /* HAVE_GTK_MULTIHEAD */
+
 static CORBA_char *
 impl_Bonobo_Control_getWindowId (PortableServer_Servant servant,
 				 const CORBA_char      *cookie,
@@ -273,12 +328,29 @@ impl_Bonobo_Control_getWindowId (PortableServer_Servant servant,
 	guint32        x11_id;
 	BonoboControl *control = BONOBO_CONTROL (
 		bonobo_object_from_servant (servant));
+#ifdef HAVE_GTK_MULTIHEAD
+	GdkScreen *gdkscreen;
+	int        screen_num;
+#endif
 
 	if (!control->priv->plug)
 		create_plug (control);
 
 	g_assert (control->priv->plug != NULL);
 
+#ifdef HAVE_GTK_MULTIHEAD
+	screen_num = parse_cookie (cookie);
+	if (screen_num != -1)
+		gdkscreen = gdk_display_get_screen (
+				gdk_display_get_default (), screen_num);
+	else
+		gdkscreen = gdk_screen_get_default ();
+
+	gtk_window_set_screen (GTK_WINDOW (control->priv->plug), gdkscreen);
+#endif
+
+	gtk_widget_show (control->priv->plug);
+
 	x11_id = gtk_plug_get_id (GTK_PLUG (control->priv->plug));
 		
 	dprintf ("plug id %d\n", x11_id);
@@ -348,11 +420,8 @@ impl_Bonobo_Control_setSize (PortableServer_Servant  servant,
 	size.height = height;
 
 	/*
-	 * Nothing.
-	 *
-	 * In the Gnome implementation of Bonobo, all size negotiation
-	 * is handled by GtkPlug/GtkSocket for us, or GtkFrame in the
-	 * local case.
+	 * In the Gnome implementation of Bonobo, all size assignment
+	 * is handled by GtkPlug/GtkSocket for us.
 	 */
 
 	g_warning ("setSize untested");
@@ -1311,6 +1380,12 @@ bonobo_control_do_popup_full (BonoboControl       *control,
 
 	g_free (path);
 
+#ifdef HAVE_GTK_MULTIHEAD
+	gtk_menu_set_screen (
+		GTK_MENU (menu),
+		gtk_window_get_screen (GTK_WINDOW (control->priv->plug)));
+#endif /* HAVE_GTK_MULTIHEAD */
+
 	gtk_widget_show (menu);
 
 	gtk_menu_popup (GTK_MENU (menu),
