@@ -97,7 +97,8 @@ ui_event (BonoboUIComponent           *component,
 			BONOBO_TYPE_UI_COMPONENT, component,
 			G_TYPE_STRING, id,
 			G_TYPE_INT, type,
-			G_TYPE_STRING, state, 0);
+			G_TYPE_STRING, state,
+			G_TYPE_INVALID);
 }
 
 static void
@@ -155,7 +156,8 @@ impl_Bonobo_UIComponent_execVerb (PortableServer_Servant servant,
 		bonobo_closure_invoke (
 			verb->closure, G_TYPE_NONE,
 			BONOBO_TYPE_UI_COMPONENT, component,
-			G_TYPE_STRING, cname, 0);
+			G_TYPE_STRING, cname,
+			G_TYPE_INVALID);
 	else
 		g_warning ("FIXME: verb '%s' not found, emit exception", cname);
 
@@ -706,6 +708,7 @@ bonobo_ui_component_set_translate (BonoboUIComponent  *component,
 		return;
 
 	node = bonobo_ui_node_from_string (xml);
+	g_return_if_fail (node != NULL);
 
 	bonobo_ui_util_translate_ui (node);
 
@@ -726,7 +729,7 @@ bonobo_ui_component_set_translate (BonoboUIComponent  *component,
  * to by @path. If @recurse then the child nodes of @path
  * are returned too, otherwise they are not.
  *
- * Return value: an XML string
+ * Return value: an XML string (CORBA allocated)
  **/
 CORBA_char *
 bonobo_ui_component_get (BonoboUIComponent *component,
@@ -1309,7 +1312,13 @@ bonobo_ui_component_set_status (BonoboUIComponent *component,
 				CORBA_Environment *opt_ev)
 {
 	if (text == NULL ||
-	    text [0] == '\0') { /* Remove what was there to reveal other msgs */
+	    text [0] == '\0') {
+		/*
+		 * FIXME: Remove what was there to reveal other msgs
+		 * NB. if we're using the same UI component as the view
+		 * was merged in with, this will result in us loosing our
+		 * status bar altogether - sub-optimal.
+		 */
 		bonobo_ui_component_rm (component, "/status/main", opt_ev);
 	} else {
 		char *str, *tmp;
